@@ -6,9 +6,11 @@ need to be stored, and as a class with a __call__ method if there are parameters
 (or statistics).
 """
 import re
-from collections import OrderedDict
-from atropos.qualtrim import quality_trim_index, nextseq_trim_index
-from atropos.align import Aligner, SEMIGLOBAL
+from .qualtrim import quality_trim_index, nextseq_trim_index
+from .align import Aligner, SEMIGLOBAL, InsertAligner
+from .util import reverse_complement
+from collections import defaultdict
+import copy
 import logging
 
 class AdapterCutter(object):
@@ -18,7 +20,7 @@ class AdapterCutter(object):
     times parameter.
     """
 
-    def __init__(self, adapters=[], times=1, action='trim', keep_match_info=False):
+    def __init__(self, adapters=[], times=1, action='trim'):
         """
         adapters -- list of Adapter objects
 
@@ -29,7 +31,6 @@ class AdapterCutter(object):
         self.times = times
         self.action = action
         self.with_adapters = 0
-        self.keep_match_info = keep_match_info
 
     def _best_match(self, read):
         """
@@ -109,6 +110,79 @@ class AdapterCutter(object):
         self.with_adapters += 1
         return trimmed_read
 
+class ReadPairModifier(object):
+    """Superclass of modifiers that edit a pair of reads simultaneously."""
+    def __call__(self, read1, read2):
+        raise NotImplemented()
+
+class InsertAdapterCutter(ReadPairModifier):
+    """
+    AdapterCutter that uses InsertAligner to first try to identify
+    insert overlap before falling back to semi-global adapter alignment.
+    """
+    def __init__(self, adapter1, adapter2, action='trim', symmetric=True, **aligner_args):
+        self.adapter1 = adapter1
+        self.adapter2 = adapter2
+        self.aligner = InsertAligner(adapter1.sequence, adapter2.sequence, **aligner_args)
+        self.action = action
+        self.symmetric = symmetric
+        self.with_adapters = [0, 0]
+    
+    def __call__(self, read1, read2):
+        match1, match2, insert_match = self.aligner.match_insert(read1.sequence, read2.sequence)
+        
+        if match1 is None and match2 is None:
+            match1 = self.adapter1.match_to(read1)
+            match2 = self.adapter2.match_to(read2)
+        
+        if self.symmetric:
+            if match1 is None and match2:
+                match1 = match2.copy()
+            if match2 is None and match1:
+                match2 = match1.copy()
+            
+        if match1 is not None:
+            match1.adapter = self.adapter1
+            match1.read = read1
+            match1.front = False
+        
+        if match2 is not None:
+            match2.adapter = self.adapter2
+            match2.read = read2
+            match2.front = False
+        
+        return (self.trim(read1, match1, 0), self.trim(read2, match2, 1))
+    
+    def trim(self, read, match, read_idx):
+        if not match:
+            read.match = None
+            read.match_info = None
+            return read
+        
+        if match.rstart < len(read):
+            trimmed_read = match.adapter.trimmed(match)
+            
+            if self.action == 'trim':
+                # read is already trimmed, nothing to do
+                pass
+            elif self.action == 'mask':
+                # add N from last modification
+                masked_sequence = trimmed_read.sequence
+                masked_sequence += ('N' * len(read) - len(trimmed_read))
+                # set masked sequence as sequence with original quality
+                trimmed_read.sequence = masked_sequence
+                trimmed_read.qualities = read.qualities
+            elif self.action is None:
+                trimmed_read = read
+        else:
+            trimmed_read = read
+        
+        trimmed_read.match = match
+        trimmed_read.match_info = [match.get_info_record()]
+        
+        self.with_adapters[read_idx] += 1
+        return trimmed_read
+
 class Trimmer(object):
     def __init__(self):
         self.trimmed_bases = 0
@@ -392,25 +466,6 @@ class SwiftBisulfiteTrimmer(object):
     def modified_bases(self):
         return self._shortCutter.trimmed_bases + self._longCutter.trimmed_bases
 
-complement = {
-    'A' : 'T',
-    'C' : 'G',
-    'R' : 'Y',
-    'S' : 'S',
-    'W' : 'W',
-    'K' : 'M',
-    'B' : 'V',
-    'D' : 'H',
-    'N' : 'N'
-}
-for k,v in list(complement.items()):
-    complement[v] = k
-    complement[k.lower()] = v.lower()
-    complement[v.lower()] = k.lower()
-
-def reverse_complement(seq):
-    return "".join(complement[base] for base in reversed(seq))
-
 class MergeOverlapping(object):
     def __init__(self, min_overlap=0.9, error_rate=0.1):
         self.min_overlap = int(min_overlap) if min_overlap > 1 else min_overlap
@@ -461,19 +516,55 @@ class MergeOverlapping(object):
 
 class Modifiers(object):
     def __init__(self, paired, merger=None):
-        self.modifiers1 = OrderedDict()
-        self.modifiers2 = OrderedDict()
+        self.modifiers = []
+        self.modifier_indexes = defaultdict(lambda: [])
         self.paired = paired
         self.merger = merger
     
     def add_modifier(self, mod_class, read=1|2, **kwargs):
-        if read & 1 > 0:
-            self.modifiers1[mod_class] = mod_class(**kwargs)
-        if read & 2 > 0 and self.paired == "both":
-            self.modifiers2[mod_class] = mod_class(**kwargs)
+        if issubclass(mod_class, ReadPairModifier):
+            assert self.paired == "both" and read == 1|2
+            mods = mod_class(**kwargs)
+        else:
+            mods = [None, None]
+            if read & 1 > 0:
+                mods[0] = mod_class(**kwargs)
+            if read & 2 > 0 and self.paired == "both":
+                mods[1] = mod_class(**kwargs)
+            if all(m is None for m in mods):
+                return None
+        return self._add_modifiers(mod_class, mods)
     
-    def get_modifier_pair(self, mod_class):
-        return [self.modifiers1.get(mod_class, None), self.modifiers2.get(mod_class, None)]
+    def add_modifier_pair(self, mod_class, read1_args=None, read2_args=None):
+        mods = [None, None]
+        if read1_args is not None:
+            mods[0] = mod_class(**read1_args)
+        if read2_args is not None and self.paired == "both":
+            mods[1] = mod_class(**read2_args)
+        if all(m is None for m in mods):
+            return None
+        return self._add_modifiers(mod_class, mods)
+    
+    def _add_modifiers(self, mod_class, mods):
+        i = len(self.modifiers)
+        self.modifiers.append(mods)
+        self.modifier_indexes[mod_class].append(i)
+        return i
+    
+    def get_modifiers(self, mod_class=None, read=None):
+        if mod_class is None:
+            mods = copy.copy(self.modifiers)
+        else:
+            mods = [self.modifiers[i] for i in self.modifier_indexes[mod_class]]
+        if read is not None:
+            mods = [m[read-1] for m in mods if m[read-1] is not None]
+        return mods
+    
+    def get_trimmer_classes(self):
+        return [
+            klass for klass in self.modifier_indexes.keys()
+            if issubclass(klass, Trimmer)
+        ]
     
     def modify(self, record):
         bp = [0,0]
@@ -481,27 +572,21 @@ class Modifiers(object):
             read1, read2 = record
             bp[0] = len(read1.sequence)
             bp[1] = len(read2.sequence)
-            for mod in self.modifiers1.values():
-                read1 = mod(read1)
-            for mod in self.modifiers2.values():
-                read2 = mod(read2)
+            for mods in self.modifiers:
+                if isinstance(mods, ReadPairModifier):
+                    read1, read2 = mods(read1, read2)
+                else:
+                    if mods[0] is not None:
+                        read1 = mods[0](read1)
+                    if mods[1] is not None:
+                        read2 = mods[1](read2)
             if self.merger:
                 read1, read2 = self.merger.merge(read1, read2)
             reads = [read1, read2]
         else:
             read = record
             bp[0] = len(read.sequence)
-            for mod in self.modifiers1.values():
-                read = mod(read)
+            for mods in self.modifiers:
+                read = mods[0](read)
             reads = [read]
         return (reads, bp)
-    
-    def __getitem__(self, i):
-        if i == 0:
-            return self.modifiers1
-        else:
-            return self.modifiers2
-    
-    def get_trimmer_classes(self):
-        return (set(k for k,v in self.modifiers1.items() if isinstance(v, Trimmer)) |
-                set(k for k,v in self.modifiers2.items() if isinstance(v, Trimmer)))
