@@ -107,22 +107,25 @@ def collect_process_statistics(N, total_bp1, total_bp2, modifiers, filters, form
     if NContentFilter in filters:
         stats["too_many_n"] = filters[NContentFilter].filtered
     
-    stats["with_adapters"] = [0, 0]
-    for i in (0, 1):
-        if AdapterCutter in modifiers[i]:
-            stats["with_adapters"][i] += modifiers[i][AdapterCutter].with_adapters
-    
-    for modifier in modifiers[0].values():
-        if isinstance(modifier, Trimmer):
-            key = "{}_bp".format(type(modifier).__name__)
-            stats[key][0] = modifier.trimmed_bases
-            
-    for modifier in modifiers[1].values():
-        if isinstance(modifier, Trimmer):
-            key = "{}_bp".format(type(modifier).__name__)
-            stats[key][1] = modifier.trimmed_bases
+    # TODO: generalize this
+    insert_cutter = modifiers.get_modifiers(InsertAdapterCutter)
+    if len(insert_cutter) > 0:
+        stats["with_adapters"] = insert_cutter[0].with_adapters
+    else:
+        stats["with_adapters"] = [0, 0]
+        for read in (1, 2):
+            for modifier in modifiers.get_modifiers(AdapterCutter, read):
+                stats["with_adapters"][read-1] += modifier.with_adapters
             
     for modifier_class in modifiers.get_trimmer_classes():
+        for modifier in modifiers.get_modifiers(modifier_class, 1):
+            key = "{}_bp".format(type(modifier).__name__)
+            stats[key][0] = modifier.trimmed_bases
+                
+        for modifier in modifiers.get_modifiers(modifier_class, 2):
+            key = "{}_bp".format(type(modifier).__name__)
+            stats[key][1] = modifier.trimmed_bases
+        
         name = modifier_class.__name__
         stats[name] = sum(stats["{}_bp".format(name)])
     
@@ -130,10 +133,17 @@ def collect_process_statistics(N, total_bp1, total_bp2, modifiers, filters, form
 
 def summarize_adapters(mods):
     summary = [{}, {}]
-    if mods[0] is not None:
-        summary[0] = collect_adapter_statistics(mods[0].adapters)
-    if mods[1] is not None:
-        summary[1] = collect_adapter_statistics(mods[1].adapters)
+    for a1, a2 in mods:
+        if a1 is not None:
+            if len(summary[0]) == 0:
+                summary[0] = collect_adapter_statistics(a1.adapters)
+            else:
+                raise Exception("Did not expect multiple read1 adapter modfiers")
+        if a2 is not None:
+            if len(summary[1]) == 0:
+                summary[1] = collect_adapter_statistics(a2.adapters)
+            else:
+                raise Exception("Did not expect multiple read2 adapter modfiers")
     return summary
 
 def collect_adapter_statistics(adapters):
