@@ -2,17 +2,10 @@
 # -*- coding: utf-8 -*-
 # kate: word-wrap off; remove-trailing-spaces all;
 
-"""
-atropos version {version}
+"""Atropos version {version}
 
 Atropos removes adapter sequences from high-throughput sequencing reads.
 
-Usage:
-    atropos -a ADAPTER [options] [-o output.fastq] input.fastq
-
-For paired-end reads:
-    atropos -a ADAPT1 -A ADAPT2 [options] -o out1.fastq -p out2.fastq in1.fastq in2.fastq
-
 Replace "ADAPTER" with the actual sequence of your 3' adapter. IUPAC wildcard
 characters are supported. The reverse complement is *not* automatically
 searched. All reads from input.fastq will be written to output.fastq with the
@@ -37,6 +30,11 @@ by John Didion, "Atropos: sensitive, specific, and speedy trimming of NGS reads,
 in prep.
 """
 
+__usage__ = """
+atropos -a ADAPTER [options] [-o output.fastq] -se input.fastq
+atropos -a ADAPT1 -A ADAPT2 [options] -o out1.fastq -p out2.fastq -pe1 in1.fastq -pe2 in2.fastq
+"""
+
 # Print a helpful error message if the extension modules cannot be imported.
 from atropos import *
 check_importability()
@@ -44,7 +42,7 @@ check_importability()
 from atropos import __version__
 from atropos.seqio import (open_reader, UnknownFileType, Formatters, RestFormatter,
                             InfoFormatter, WildcardFormatter, FormatError, Writers)
-from atropos.adapters import AdapterParser
+from atropos.adapters import AdapterParser, BACK
 from atropos.modifiers import *
 from atropos.filters import *
 from atropos.report import *
@@ -55,12 +53,14 @@ import copy
 import logging
 import os
 import platform
+import re
 import sys
 import time
+import textwrap
 
 def main(cmdlineargs=None, default_outfile="-"):
     """
-    Main function that evaluates command-line parameters and iterates over all reads.
+    Main function that evaluates command-line parameters.
     
     :param cmdlineargs: iterable of command line arguments; `None` is equivalent to
     `sys.argv[1:]`
@@ -77,8 +77,25 @@ def main(cmdlineargs=None, default_outfile="-"):
         level = options.log_level or ("ERROR" if options.quiet else "INFO")
         setup_logging(stdout=bool(options.output), level=level)
     
+    logger = logging.getLogger()
+    logger.info("This is Atropos %s with Python %s", __version__, platform.python_version())
+    logger.info("Command line parameters: %s", " ".join(orig_args))
+    
+    # dispatch to subcommands if one is specified
+    if options.command == "detect":
+        #try:
+        import atropos.detect
+        atropos.detect.main(options)
+        #except:
+        #    parser.error("Error during adapter detection; do you have khmer installed?")
+        #    sys.exit(1)
+    # otherwise trim reads
+    else:
+        run_atropos(options, parser, default_outfile="-")
+
+def run_atropos(options, parser, default_outfile="-"):
     paired = validate_options(options, parser)
-    reader, qualities, has_qual_file = create_reader(options.inputs, options, paired, parser)
+    reader, qualities, has_qual_file = create_reader(options, paired, parser)
     modifiers, num_adapters = create_modifiers(options, paired, qualities, has_qual_file, parser)
     min_affected = 2 if options.pair_filter == 'both' else 1
     filters = create_filters(options, paired, min_affected)
@@ -86,13 +103,10 @@ def main(cmdlineargs=None, default_outfile="-"):
     writers = Writers(force_create)
     
     logger = logging.getLogger()
-    logger.info("This is atropos %s with Python %s", __version__, platform.python_version())
-    logger.info("Command line parameters: %s", " ".join(orig_args))
     logger.info("Trimming %s adapter%s with at most %.1f%% errors in %s mode ...",
         num_adapters, 's' if num_adapters > 1 else '', options.error_rate * 100,
         { False: 'single-end', 'first': 'paired-end legacy', 'both': 'paired-end' }[paired])
-    if paired == 'first' and (len(modifiers.modifiers2) > 0 or options.quality_cutoff):
-        import textwrap
+    if paired == 'first' and (len(modifiers.get_modifiers(read=2)) > 0 or options.quality_cutoff):
         logger.warning('\n'.join(textwrap.wrap('WARNING: Requested read '
             'modifications are applied only to the first '
             'read since backwards compatibility mode is enabled. '
@@ -125,24 +139,52 @@ def main(cmdlineargs=None, default_outfile="-"):
         stop_cpu_time - start_cpu_time,
         summary,
         modifiers.get_trimmer_classes())
-    
+
+class ParagraphHelpFormatter(argparse.HelpFormatter):
+    def _fill_text(self, text, width, indent):
+        text = re.sub('[ \t]{2,}', ' ', text)
+        paragraphs = [
+            textwrap.fill(p, width, initial_indent=indent, subsequent_indent=indent)
+            for p in re.split("\n\n", text)
+        ]
+        return "\n\n".join(paragraphs)
+
 def get_argument_parser():
-    parser = argparse.ArgumentParser(usage=__doc__.lstrip().format(version=__version__))
-    parser.add_argument("inputs", nargs=argparse.REMAINDER)
+    parser = argparse.ArgumentParser(
+        usage=__usage__,
+        description=__doc__.format(version=__version__),
+        formatter_class=ParagraphHelpFormatter)
     
     parser.add_argument("--debug", action='store_true', default=False,
         help="Print debugging information. (no)")
-    parser.add_argument("-f", "--format",
+    parser.add_argument("--progress", default=None,
+        help="Show progress. bar = show progress bar; msg = show a status message. (no)")
+    parser.add_argument("--op-order", default="CGQA",
+        help="The order in which trimming operations are be applied. This is a string of "
+             "1-4 of the following characters: A = adapter trimming; C = cutting "
+             "(unconditional); G = NextSeq trimming; Q = quality trimming. The default is "
+             "'CGQA' to maintain compatibility with Cutadapt; however, this is likely to "
+             "change to 'GACQ' in the near future.")
+    
+    group = parser.add_argument_group("Input")
+    group.add_argument("-pe1", "--input1", default=None, metavar="FILE1",
+        help="The first (and possibly only) input file.")
+    group.add_argument("-pe2", "--input2",  default=None, metavar="FILE2",
+        help="The second input file.")
+    group.add_argument("-se", "--single-input", default=None, metavar="FILE",
+        help="A single-end read file.")
+    group.add_argument("-sq", "--single-quals", default=None, metavar="FILE",
+        help="A single-end qual file.")
+    group.add_argument("-l", "--interleaved-input", default=None, metavar="FILE",
+        help="Interleaved input file.")
+    group.add_argument("-f", "--format",
         help="Input file format; can be either 'fasta', 'fastq' or 'sra-fastq'. "
             "Ignored when reading csfasta/qual files. (auto-detect "
             "from file name extension)")
-    parser.add_argument("--max-reads", default=None,
+    group.add_argument("--max-reads", default=None,
         help="Maximum number of reads/pairs to process (no max)")
-    parser.add_argument("--progress", default=None,
-        help="Show progress. bar = show progress bar; msg = show a status "
-             "message. (no)")
     
-    group = parser.add_argument_group("Finding adapters:",
+    group = parser.add_argument_group("Finding adapters",
         description="Parameters -a, -g, -b specify adapters to be removed from "
             "each read (or from the first read in a pair if data is paired). "
             "If specified multiple times, only the best matching adapter is "
@@ -169,6 +211,13 @@ def get_argument_parser():
             "as with -a. This option is mostly for rescuing failed library "
             "preparations - do not use if you know which end your adapter was "
             "ligated to! (none)")
+    group.add_argument("--aligner", choices=('adapter', 'insert'), default='adapter',
+        help="Which alignment algorithm to use for identifying adapters. Currently, "
+             "you can choose between the semi-global alignment strategy used in Cutdapt "
+             "('adapter') or the more accurate insert-based alignment algorithm ('insert'). "
+             "Note that insert-based alignment can only be used with paired-end reads "
+             "containing 3' adapters. New algorithms are being implemented and the default "
+             "is likely to change. (adapter)")
     group.add_argument("-e", "--error-rate", type=float, default=None,
         help="Maximum allowed error rate (no. of errors divided by the length "
             "of the matching region). (0.1)")
@@ -202,6 +251,7 @@ def get_argument_parser():
              "it specifies the minimum length as the fraction of the length of the *shorter* read "
              "in the pair; otherwise it specifies the minimum number of overlapping base pairs ("
              "with an absolute minimum of 2 bp). (0.9)")
+    # TODO: error correction:
     #group.add_argument("--merge-mismatches", choices=("best", "N", "random"), default="best",
     #	help="How to handle mismatches while merging; best: select the base with the best quality; "
     #		 "N: set the base to N; random: select one of the two bases at random. Note that if "
@@ -349,14 +399,14 @@ def get_argument_parser():
              "(see --cut-min). (no)")
     group.add_argument("-p", "--paired-output", metavar="FILE",
         help="Write second read in a pair to FILE. (no)")
+    group.add_argument("-L", "--interleaved-output", metavar="FILE",
+        help="Write output to interleaved file.")
     # Setting the default for pair_filter to None allows us to find out whether
     # the option was used at all.
     group.add_argument("--pair-filter", metavar='(any|both)', default=None,
         choices=("any", "both"),
         help="Which of the reads in a paired-end read have to match the "
             "filtering criterion in order for it to be filtered. (any)")
-    group.add_argument("--interleaved", action='store_true', default=False,
-        help="Read and write interleaved paired-end reads. (no)")
     group.add_argument("--untrimmed-paired-output", metavar="FILE",
         help="Write second read in a pair to this FILE when no adapter "
             "was found in the first read. Use this option together with "
@@ -404,7 +454,25 @@ def get_argument_parser():
         help="Size of queue for batches of results to be written. (THREADS * 100)")
     group.add_argument("--compression", choices=("worker", "writer"), default=None,
         help="Where data compression should be performed. Defaults to 'writer' if "
-             "system-level compression can be used, otherwise defaults to 'worker'.")
+             "system-level compression can be used and (1 < threads < 8), otherwise "
+             "defaults to 'worker'.")
+    
+    # add subcommands
+    parser.set_defaults(command=None)
+    subparsers = parser.add_subparsers(
+        dest="commands",
+        help='sub-command help')
+    
+    detect_parser = subparsers.add_parser("detect", help="Detect adapter sequences",
+        usage="\natropos -se input.fastq detect\natropos -pe1 in1.fastq -pe2 in2.fastq detect",
+        description="Detect adapter sequences directly from read sequences.")
+    detect_parser.set_defaults(command='detect', max_reads=10000)
+    detect_parser.add_argument("--kmer-size", type=int, default=12,
+        help="Size of k-mer used to scan reads for adapter sequences. (12)")
+    detect_parser.add_argument("--max-adapters", type=int, default=None,
+        help="The maximum number of candidate adapters to report. (10000)")
+    detect_parser.add_argument("--adapter-report", default="-",
+        help="File in which to write the summary of detected adapters. (stdout)")
     
     return parser
 
@@ -434,42 +502,24 @@ def validate_options(options, parser):
     if options.debug and options.threads is not None:
         parser.error("Cannot use debug mode with multiple threads")
     
-    args = options.inputs
-    if len(args) == 0:
-        parser.error("At least one parameter needed: name of a FASTA or FASTQ file.")
-    elif len(args) > 2:
-        parser.error("Too many parameters.")
-    if args[0].endswith('.qual'):
-        parser.error("If a .qual file is given, it must be the second argument.")
-    
     # Find out which 'mode' we need to use.
-    # Default: single-read trimming (neither -p nor -A/-G/-B/-U/--interleaved given)
     paired = False
-    if options.paired_output:
-        # Modify first read only, keep second in sync (-p given, but not -A/-G/-B/-U).
-        # This exists for backwards compatibility ('legacy mode').
-        paired = 'first'
-    # Any of these options switch off legacy mode
-    if (options.adapters2 or options.front2 or options.anywhere2 or
-        options.cut2 or options.cut_min2 or options.interleaved or options.pair_filter or
-        options.too_short_paired_output or options.too_long_paired_output):
-        # Full paired-end trimming when both -p and -A/-G/-B/-U given
-        # Read modifications (such as quality trimming) are applied also to second read.
-        paired = 'both'
-
-    if paired and len(args) == 1 and not options.interleaved:
-        parser.error("When paired-end trimming is enabled via -A/-G/-B/-U or -p, "
-            "two input files are required.")
-    if options.interleaved and len(args) != 1:
-        parser.error("When reading interleaved files, only one input file may "
-            "be given.")
-    if not paired:
+    if options.single_input:
+        if not options.output:
+            parser.error("An output file is required")
+        if options.input1 or options.input2 or options.interleaved_input:
+            parser.error("Cannot use -se together with -pe1, -pe2, or -l")
         if options.untrimmed_paired_output:
             parser.error("Option --untrimmed-paired-output can only be used when "
                 "trimming paired-end reads (with option -p).")
-
-    if paired:
-        if not options.interleaved:
+        options.input1 = options.single_input
+        options.input2 = options.single_quals
+    else:
+        if not options.interleaved_input and (not options.input1 or not options.input2):
+            parser.error("Both '-pe1' and '-pe2' are required for paired-end trimming. If this is an "
+                "interleaved file, use '-l' instead.")
+        
+        if not options.interleaved_output:
             if not options.paired_output:
                 parser.error("When paired-end trimming is enabled via -A/-G/-B/-U, "
                     "a second output file needs to be specified via -p (--paired-output).")
@@ -485,10 +535,24 @@ def validate_options(options, parser):
             if options.too_long_output and not options.too_long_paired_output:
                 parser.error("When using --too-long-output with paired-end "
                     "reads, you also need to use --too-long-paired-output")
-    elif len(args) == 2:
-        if options.format is not None:
-            parser.error("If a pair of .fasta and .qual files is given, the -f/--format "
-                "parameter cannot be used.")
+                
+        # Modify first read only, keep second in sync (-p given, but not -A/-G/-B/-U).
+        # This exists for backwards compatibility ('legacy mode').
+        paired = 'first'
+    
+        # Any of these options switch off legacy mode
+        if (options.adapters2 or options.front2 or options.anywhere2 or options.cut2 or
+            options.cut_min2 or options.interleaved_input or options.pair_filter or
+            options.too_short_paired_output or options.too_long_paired_output):
+            # Full paired-end trimming when both -p and -A/-G/-B/-U given
+            # Read modifications (such as quality trimming) are applied also to second read.
+            paired = 'both'
+
+    if options.aligner != 'adapter':
+        if options.aligner == 'insert' and paired != 'both':
+            parser.error("Insert aligner only works with paired-end reads")
+            # TODO: should also be checking that there is exactly one 3' adapter for each read
+            # TODO: have the aligner tell us whether it can be used based on options?
     
     if options.merge_overlapping and (
             paired != "both" or
@@ -501,6 +565,10 @@ def validate_options(options, parser):
         parser.error("The input file format must be either 'fasta', 'fastq' or "
             "'sra-fastq' (not '{0}').".format(options.format))
     
+    options.op_order = list(options.op_order)
+    if not (1 <= len(options.op_order) <= 4 and all(o in ('A','C','G','Q') for o in options.op_order)):
+        parser.error("--op-order must be a string of 1-4 characters consisting of A,C,G,Q")
+    
     if options.mirna and options.bisulfite is not None:
         parser.error("Can only specify one of --mirna, --bisulfite")
     if options.mirna:
@@ -654,15 +722,6 @@ def validate_options(options, parser):
     options.max_reads = int_or_str(options.max_reads)
     
     if options.threads is not None:
-        if options.compression is None:
-            if options.no_writer_process:
-                options.compression = "worker"
-            else:
-                from atropos.compression import can_use_system_compression
-                options.compression = "writer" if can_use_system_compression() else "worker"
-        elif options.compression == "writer" and options.no_writer_process:
-            parser.error(" writer compression and --no-writer-process are mutually exclusive")
-        
         threads = options.threads
         if threads <= 0:
             threads = cpu_count()
@@ -670,6 +729,20 @@ def validate_options(options, parser):
             parser.error("At least two threads are required for multi-processing")
         options.threads = threads
         
+        if options.compression is None:
+            # Our tests show that with 8 or more threads, worker compression is
+            # more efficient.
+            if options.no_writer_process or threads == 2 or threads >= 8:
+                options.compression = "worker"
+            else:
+                from atropos.compression import can_use_system_compression
+                options.compression = "writer" if can_use_system_compression() else "worker"
+        elif options.compression == "writer" and options.no_writer_process:
+            parser.error("Writer compression and --no-writer-process are mutually exclusive")
+        elif threads == 2 and options.compression == "writer":
+            logging.getLogger.warn("Writer compression requires > 2 threads; using worker compression instead")
+            options.compression = "worker"
+        
         options.batch_size = int_or_str(options.batch_size)
         if options.process_timeout < 0:
             options.process_timeout = 0
@@ -689,21 +762,23 @@ def validate_options(options, parser):
     
     return paired
 
-def create_reader(input_files, options, paired, parser, counter_magnitude="M"):
-    input_filename = input_files[0]
-    input_paired_filename = None
-    quality_filename = None
-    if len(input_files) > 1:
+def create_reader(options, paired, parser, counter_magnitude="M"):
+    input1 = input2 = qualfile = None
+    interleaved = False
+    if options.interleaved_input:
+        input1 = options.interleaved_input
+        interleaved = True
+    else:
+        input1 = options.input1
         if paired:
-            input_paired_filename = input_files[1]
+            input2 = options.input2
         else:
-            quality_filename = input_files[1]
+            qualfile = options.input2
     
     try:
-        reader = open_reader(
-            input_filename, file2=input_paired_filename,
-            qualfile=quality_filename, colorspace=options.colorspace,
-            fileformat=options.format, interleaved=options.interleaved)
+        reader = open_reader(input1, file2=input2, qualfile=qualfile,
+            colorspace=options.colorspace, fileformat=options.format,
+            interleaved=interleaved)
     except (UnknownFileType, IOError) as e:
         parser.error(e)
     
@@ -719,7 +794,7 @@ def create_reader(input_files, options, paired, parser, counter_magnitude="M"):
     elif options.progress == "bar":
         reader = create_progress_reader(reader, options.max_reads, counter_magnitude)
     
-    return (reader, qualities, quality_filename is not None)
+    return (reader, qualities, qualfile is not None)
 
 class BatchIterator(object):
     def __init__(self, reader, size, max_reads=None):
@@ -874,26 +949,6 @@ def create_tqdm_reader(reader):
     import tqdm
     return tqdm.tqdm(reader)
 
-def create_filters(options, paired, min_affected):
-    filters = Filters(FilterFactory(paired, min_affected))
-    
-    if options.minimum_length is not None and options.minimum_length > 0:
-        filters.add_filter(TooShortReadFilter, options.minimum_length)
-    
-    if options.maximum_length < sys.maxsize:
-        filters.add_filter(TooLongReadFilter, options.maximum_length)
-    
-    if options.max_n >= 0:
-        filters.add_filter(NContentFilter, options.max_n)
-    
-    if options.discard_trimmed:
-        filters.add_filter(TrimmedFilter)
-    
-    if options.discard_untrimmed or options.untrimmed_output:
-        filters.add_filter(UntrimmedFilter)
-    
-    return filters
-
 def create_modifiers(options, paired, qualities, has_qual_file, parser):
     adapter_parser = AdapterParser(
         colorspace=options.colorspace,
@@ -923,6 +978,11 @@ def create_modifiers(options, paired, qualities, has_qual_file, parser):
             options.max_n == -1 and not options.trim_n:
         parser.error("You need to provide at least one adapter sequence.")
     
+    if options.aligner == 'insert' and (
+            not adapters1 or len(adapters1) > 1 or adapters1[0].where != BACK or
+            not adapters2 or len(adapters2) > 1 or adapters2[0].where != BACK):
+        parser.error("Insert aligner requires a single 3' adapter for each read")
+    
     if options.debug:
         for adapter in adapters1 + adapters2:
             adapter.enable_debug()
@@ -935,21 +995,30 @@ def create_modifiers(options, paired, qualities, has_qual_file, parser):
     
     modifiers = Modifiers(paired, merger)
     
-    if options.cut or options.cut2:
-        modifiers.add_modifier(UnconditionalCutter, read=1, lengths=options.cut)
-        modifiers.add_modifier(UnconditionalCutter, read=2, lengths=options.cut2)
-    
-    if options.nextseq_trim is not None:
-        modifiers.add_modifier(NextseqQualityTrimmer, read=1,
-                               cutoff=options.nextseq_trim, base=options.quality_base)
-    
-    if options.quality_cutoff:
-        modifiers.add_modifier(QualityTrimmer, cutoff_front=options.quality_cutoff[0],
-                               cutoff_back=options.quality_cutoff[1], base=options.quality_base)
-    
-    if adapters1 or adapters2:
-        modifiers.add_modifier(AdapterCutter, read=1, adapters=adapters1, times=options.times, action=options.action)
-        modifiers.add_modifier(AdapterCutter, read=2, adapters=adapters2, times=options.times, action=options.action)
+    for op in options.op_order:
+        if op == 'A' and (adapters1 or adapters2):
+            # TODO: generalize this using some kind of factory class
+            if options.aligner == 'insert':
+                modifiers.add_modifier(InsertAdapterCutter,
+                    adapter1=adapters1[0], adapter2=adapters2[0], action=options.action)
+            else:
+                modifiers.add_modifier_pair(AdapterCutter,
+                    dict(adapters=adapters1, times=options.times, action=options.action),
+                    dict(adapters=adapters2, times=options.times, action=options.action)
+                )
+        elif op == 'C' and (options.cut or options.cut2):
+            modifiers.add_modifier_pair(UnconditionalCutter,
+                dict(lengths=options.cut),
+                dict(lengths=options.cut2)
+            )
+        elif op == 'G' and (options.nextseq_trim is not None):
+            modifiers.add_modifier(NextseqQualityTrimmer,
+                read=1, cutoff=options.nextseq_trim, base=options.quality_base)
+        elif op == 'Q' and options.quality_cutoff:
+            modifiers.add_modifier(QualityTrimmer,
+                cutoff_front=options.quality_cutoff[0],
+                cutoff_back=options.quality_cutoff[1],
+                base=options.quality_base)
     
     if options.bisulfite:
         if isinstance(options.bisulfite, str):
@@ -972,8 +1041,10 @@ def create_modifiers(options, paired, qualities, has_qual_file, parser):
         modifiers.add_modifier(NEndTrimmer)
     
     if options.cut_min or options.cut_min2:
-        modifiers.add_modifier(MinCutter, read=1, lengths=options.cut_min)
-        modifiers.add_modifier(MinCutter, read=2, lengths=options.cut_min2)
+        modifiers.add_modifier_pair(MinCutter,
+            dict(lengths=options.cut_min),
+            dict(lengths=options.cut_min2)
+        )
     
     if options.length_tag:
         modifiers.add_modifier(LengthTagModifier, length_tag=options.length_tag)
@@ -995,13 +1066,42 @@ def create_modifiers(options, paired, qualities, has_qual_file, parser):
     
     return (modifiers, len(adapters1) + len(adapters2))
 
+def create_filters(options, paired, min_affected):
+    filters = Filters(FilterFactory(paired, min_affected))
+    
+    if options.minimum_length is not None and options.minimum_length > 0:
+        filters.add_filter(TooShortReadFilter, options.minimum_length)
+    
+    if options.maximum_length < sys.maxsize:
+        filters.add_filter(TooLongReadFilter, options.maximum_length)
+    
+    if options.max_n >= 0:
+        filters.add_filter(NContentFilter, options.max_n)
+    
+    if options.discard_trimmed:
+        filters.add_filter(TrimmedFilter)
+    
+    if options.discard_untrimmed or options.untrimmed_output:
+        filters.add_filter(UntrimmedFilter)
+    
+    return filters
+
 def create_formatters(options, qualities, default_outfile):
+    output1 = output2 = None
+    interleaved = False
+    if options.interleaved_output:
+        output1 = options.interleaved_output
+        interleaved = True
+    else:
+        output1 = options.output
+        output2 = options.paired_output
+    
     seq_formatter_args = dict(
         qualities=qualities,
         colorspace=options.colorspace,
-        interleaved=options.interleaved
+        interleaved=interleaved
     )
-    formatters = Formatters(options.output, seq_formatter_args)
+    formatters = Formatters(output1, seq_formatter_args)
     force_create = []
         
     if (options.merge_overlapping and options.merged_output):
@@ -1018,13 +1118,12 @@ def create_formatters(options, qualities, default_outfile):
             options.too_long_output, options.too_long_paired_output)
     
     if not formatters.multiplexed:
-        if options.output is not None:
-            formatters.add_seq_formatter(NoFilter,
-                options.output, options.paired_output)
-            if options.output != "-":
-                force_create.append(options.output)
-                if options.paired_output is not None:
-                    force_create.append(options.paired_output)
+        if output1 is not None:
+            formatters.add_seq_formatter(NoFilter, output1, output2)
+            if output1 != "-":
+                force_create.append(output1)
+                if output2 is not None:
+                    force_create.append(output2)
         elif not (options.discard_trimmed and options.untrimmed_output):
             formatters.add_seq_formatter(NoFilter, default_outfile)
             if default_outfile != "-":
@@ -1032,7 +1131,7 @@ def create_formatters(options, qualities, default_outfile):
     
     if not options.discard_untrimmed:
         if formatters.multiplexed:
-            untrimmed = options.untrimmed_output or options.output.format(name='unknown')
+            untrimmed = options.untrimmed_output or output1.format(name='unknown')
             formatters.add_seq_formatter(UntrimmedFilter, untrimmed)
             formatters.add_seq_formatter(NoFilter, untrimmed)
         elif options.untrimmed_output:
@@ -1069,7 +1168,7 @@ def run_serial(reader, modifiers, filters, formatters, writers):
         
         return (0, Summary(
             collect_process_statistics(n, total_bp1, total_bp2, modifiers, filters, formatters),
-            summarize_adapters(modifiers.get_modifier_pair(AdapterCutter)),
+            summarize_adapters(modifiers.get_modifiers(AdapterCutter)),
             modifiers.get_trimmer_classes()
         ).finish())
     
@@ -1081,7 +1180,7 @@ def run_serial(reader, modifiers, filters, formatters, writers):
             return (1, None)
         raise
     except (FormatError, EOFError) as e:
-        logging.getLogger().error("Cutadapt error", exc_info=True)
+        logging.getLogger().error("Atropos error", exc_info=True)
         return(1, None)
     finally:
         reader.close()
